unit DBScrollBox;  { TEtwDBScrollBox component. }

interface

uses
  Windows, 
  SysUtils, 
  Messages, 
  Classes, 
  Graphics, 
  Controls, 
  Forms, 
  Dialogs, 
  DBTables, 
  DB, 
  DBCtrls, 
  StdCtrls, 
  wwDBEdit, 
  wwDatSrc, 
  wwRiched, 
  DsgnIntf, 
  Menus, 
  IniFiles, 
  Wwdbdlg, 
  wwdblook, 
  Wwdotdot, 
  Wwdbspin, 
  Wwdbcomb, 
  Mask, 
  EtWAbout;

CONST
  VERSION = 'Version 5.6';

type

  TEtWDefaultEditor = class(TDefaultEditor)
    private
      FEventToEdit : string;
      FEventEditor : TPropertyEditor;
    protected
      procedure CheckPropertyEditor(PropertyEditor: TPropertyEditor); dynamic;
    public
      procedure Edit; override;
      property  EventToEdit: string read  FEventToEdit
      write FEventToEdit;
  end;

  TEtwPrefix = class(TPersistent)
    private
      FEditFields           : string;
      FMemoFields           : string;
      FImageFields          : string;
      FLabels               : string;
      FComboBoxFields       : string;
      FSpinEditFields       : string;
      FComboDlgFields       : string;
      FLookupComboFields    : string;
      FLookupComboDlgFields : string;
      FCheckboxFields       : string;
      FPrefixOwner          : TComponent;

      procedure SetEditFields           (aString : string);
      procedure SetMemoFields           (aString : string);
      procedure SetImageFields          (aString : string);
      procedure SetLabels               (aString : string);
      procedure SetComboBoxFields       (aString : string);
      procedure SetSpinEditFields       (aString : string);
      procedure SetComboDlgFields       (aString : string);
      procedure SetLookupComboFields    (aString : string);
      procedure SetLookupComboDlgFields (aString : string);
      procedure SetCheckBoxFields       (aString : string);
      function  ValidPrefix    (aString : string) : boolean;
    public
      constructor CreateFor (MyComponent : TComponent);
    published
      property EditFields           : string read  FEditFields            write SetEditFields;
      property MemoFields           : string read  FMemoFields            write SetMemoFields;
      property ImageFields          : string read  FImageFields           write SetImageFields;
      property Labels               : string read  FLabels                write SetLabels;
      property ComboBoxFields       : string read  FComboBoxFields        write SetComboBoxFields;
      property SpinEditFields       : string read  FSpinEditFields        write SetSpinEditFields;
      property ComboDlgFields       : string read  FComboDlgFields        write SetComboDlgFields;
      property LookupComboFields    : string read  FLookupComboFields     write SetLookupComboFields;
      property LookupComboDlgFields : string read  FLookupComboDlgFields  write SetLookupComboDlgFields;
      property CheckboxFields       : string read  FCheckboxFields        write SetCheckboxFields;
  end;

  TEtwLabelAlign = (alTop,alLeft);

  TEtwDBScrollBox = class(TScrollBox)
    private
      { Private declarations }
      FAboutInfo             : TEtWAboutInfo;
      FFieldsList            : TList;
      FControlList           : TList;
      FDatasource            : TwwDatasource;
      FMayEditProperties     : boolean;
      FConsistentEditWidth   : boolean;
      FLabelStripUnderscores : boolean;
      FLabelMixedCase        : boolean;
      FLeftAlignmentTFields  : boolean;
      FLabelSpace            : integer;
      FCharWidth             : integer;
      FSemiColonWidth        : integer;
      FEditSpacing           : integer;
      FLabelUseSemiColon     : boolean;
      FLabelAlignment        : TAlignment;
      FLabelAlign            : TEtwLabelAlign;
      FPrefix                : TEtwPrefix;
      FRequiredColor         : TColor;
      FComingFromTheEditor   : boolean;
      FNumberOfEditorChanges : integer;
      procedure Loaded; override;
      procedure ReadIniFile;
      procedure SetEditSpacing (aInteger : integer);
      procedure SetLabelStripUnderscores (aBoolean : Boolean);
      procedure SetLabelUseSemiColon (aBoolean: Boolean);
      procedure SetLabelAlignment(aAlignment : TAlignment);
      procedure SetLabelAlign (aEtwLabelAlign : TEtwLabelAlign);
      procedure SetLabelSpace  (aInteger : integer);
      procedure SetConsistentEditWidth (aBoolean : boolean);
      procedure SetRequiredColor (aColor : TColor);
      procedure SetLeftAlignmentTFields (aBoolean : boolean);
      procedure SetLabelMixedCase (aBoolean : boolean);
      function  GetDataSource: TwwDatasource;  { Returns linked data source. }
      procedure SetDataSource(newSource: TwwDatasource);  { Assigns new data source. }
    protected
      { Protected declarations }
      procedure AlignLabelsAndFields;
      function  CreateCaption ( aInteger : integer) : string;
      procedure CreateUniqueName (aControl : TControl; prefix : string; i : integer);
    public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor  Destroy; override;
      procedure   notification (AComponent : TComponent; Operation : TOperation); override;
    published
      { Published properties and events }
      property About                  : TEtWAboutInfo    read FAboutInfo
      write FAboutInfo
      stored False;

      property DataSource             : TwwDatasource   read  GetDataSource
      write SetDataSource;
      property LabelUseSemiColon      : boolean         read  FLabelUseSemiColon
      write SetLabelUseSemiColon default false;
      property ConsistentEditWidth    : boolean         read  FConsistentEditWidth
      write SetConsistentEditWidth default false;
      property EditSpacing            : integer         read  FEditSpacing
      write SetEditSpacing default 2;
      property Prefix                 : TEtwPrefix      read  FPrefix
      write FPrefix;
      property LabelAlignment         : TAlignment      read  FLabelAlignment
      write SetLabelAlignment default taLeftJustify;
      property LabelAlign             : TEtwLabelAlign  read  FLabelAlign
      write SetLabelAlign default alLeft;
      property LabelStripUnderscores  : boolean         read  FLabelStripUnderscores
      write SetLabelStripUnderscores default false;
      property LabelSpace             : integer         read  FLabelspace
      write SetLabelspace default 4;
      property LeftAlignmentTFields   : boolean         read  FLeftAlignmentTFields
      write SetLeftAlignmentTFields default false;
      property RequiredColor          : TColor          read  FRequiredColor
      write SetRequiredColor default clWindow;
      property LabelMixedCase         : boolean         read  FLabelMixedCase
      write SetLabelMixedCase default false;

      property MayEditProperties      : boolean         read  FMayEditProperties;
  end;  { TEtwDBScrollBox }

  TEtwDBScrollBoxEditor = class(TEtWDefaultEditor)
    public
      function GetVerbCount: integer; override;
      function GetVerb(Index: integer): string; override;
      procedure ExecuteVerb(Index: integer); override;
  end;  { TEtwDBScrollBoxEditor }

procedure Register;

implementation
{$R INFO32.RES}

uses
  WinProcs, TypInfo, edtDBScrollBox, edtDBScrollBoxOrd;

procedure TEtWDefaultEditor.CheckPropertyEditor(PropertyEditor: TPropertyEditor);
{ Checks PropertyEditor to make sure it matches FEventToEdit. }
var
  localFreeEditor : Boolean;
begin
  localFreeEditor := True;
  try
    if not Assigned(FEventEditor) and (PropertyEditor is TMethodProperty) then
      begin
        if CompareText(PropertyEditor.GetName, FEventToEdit) = 0 then
          begin
            localFreeEditor := False;		{ Don't free since we're going to hold on to this and free later. }
            FEventEditor    := PropertyEditor;
          end;
      end;
  finally
    if localFreeEditor then
        PropertyEditor.Free;
  end;
end;		{ CheckPropertyEditor }

procedure TEtWDefaultEditor.Edit;

var
  Components : TComponentList;
begin
  if FEventToEdit = '' then
    begin
      { no event to edit specified, so use inherited behavior. }
      inherited Edit;
      exit;
    end;

  Components := TComponentList.Create;

  try
    Components.Add(Component);
    FEventEditor := nil;
    try
      GetComponentProperties(Components, tkAny, Designer, CheckPropertyEditor);
      if Assigned(FEventEditor) then
        begin
          FEventEditor.Edit;
        end;
    finally
      FEventEditor.Free;
    end;		{ try/finally }
  finally
    Components.Free;
  end;		{ try/finally }

end;		{ Edit }

constructor TEtwPrefix.CreateFor (MyComponent : TComponent);

begin
  inherited create;
  FPrefixOwner := MyComponent;
end;

function TEtwDBScrollBoxEditor.GetVerbCount: integer;
begin
  result := 5;
end;  { GetVerbCount }

function TEtwDBScrollBoxEditor.GetVerb(Index: integer): string;
begin
  case Index of
    0:
        result := 'Change fieldtypes';
    1:
        result := 'Control order';
    2:
        result := 'Save TEtwDBScrollBox properties';
    3:
        result := 'Read TEtwDBScrollBox properties';
    4:
        result := 'Change TEtwDBScrollBox to TScrollbox';
  end;
  { case }
end;  { GetVerb }

procedure TEtwDBScrollBoxEditor.ExecuteVerb(Index: integer);
var
  Ini          : TIniFile;
  DirectoryArr : array [0..255] of char;
  Directory    : string;
  i, j         : integer;
  Reference    : TControlClass;
  Instance     : TControl;
  DataSource   : TwwDataSource;
  DataField    : String;
  Color        : TColor;
  ScrollBox    : TScrollBox;
  Order        : Integer;
begin

  Datasource := nil;
  Color      := clWindow;
  Reference  := nil;

  case Index of

    0:
      begin
        if not TEtwDBScrollBox(Component).MayEditProperties then
          begin
            raise exception.create('Overwriting fieldtypes at this stage is not allowed');
          end;

        try

          frmEdtDBScrollBox := TfrmEdtDBScrollBox.create(application);

          for i := 0 to TEtwDBScrollBox(Component).FControlList.Count - 1 do
            begin
              if not (TControl(TEtwDBScrollBox(Component).FControlList.items[i]) is Tlabel) then
                begin
                  frmEdtDBScrollBox.AddItem(TControl(TEtwDBScrollBox(Component).FControlList[i]));
                end;
            end;

          if frmEdtDBScrollBox.lbxDBScrollBox.Items.Count = 0 then
            begin
              raise exception.create('Overwriting fieldtypes at this stage is not allowed');
            end;

          if frmEdtDBScrollBox.ShowModal = mrOk then
            begin
              TEtwDBScrollBox(Component).FComingFromTheEditor := true;

              if frmEdtDBScrollBox.lbxDBScrollBox.Selcount > 0 then
                begin
                  TEtwDBScrollBox(Component).FNumberOfEditorChanges := frmEdtDBScrollBox.lbxDBScrollBox.Selcount;

                  for i := 0 to (frmEdtDBScrollBox.lbxDBScrollBox.Items.Count - 1) do
                    begin
                      if frmEdtDBScrollBox.lbxDBScrollBox.Selected[i] then
                        begin
                          j := (i * 2) + 1; // entry in FControlList;

                          case frmEdtDBScrollBox.rgpChangeTo.ItemIndex of
                            0 :
                                Reference := TControlClass(TwwDBEdit);            // TwwDBCustomEdit
                            1 :
                                Reference := TControlClass(TwwDBComboBox);        // TwwDBCustomEdit
                            2 :
                                Reference := TControlClass(TwwDBSpinEdit);        // TwwDBCustomEdit
                            3 :
                                Reference := TControlClass(TwwDBComBoDlg);        // TwwDBCustomLookupCombo
                            4 :
                                Reference := TControlClass(TwwDBLookupCombo);     // TwwDBCustomLookupCombo
                            5 :
                                Reference := TControlClass(TwwDBLookupComboDlg);  // TwwDBCustomLookupCombo
                            6 :
                                Reference := TControlClass(TwwDBRichEdit);        // TwwDBRichEdit
                            7 :
                                Reference := TControlClass(TDBImage);             // TDBImage
                            8 :
                                Reference := TControlClass(TDBMemo);              // TDBMemo
                            9 :
                                Reference := TControlClass(TDBCheckBox);              // TDBMemo
                          end;

                          Instance         := Reference.create(Component.Owner);
                          Instance.Visible := false;

                          Instance.Left   := TControl(TEtwDBScrollBox(Component).FControlList[j]).left;
                          Instance.Top    := TControl(TEtwDBScrollBox(Component).FControlList[j]).Top;
                          Instance.Height := TControl(TEtwDBScrollBox(Component).FControlList[j]).Height;
                          Instance.Width  := TControl(TEtwDBScrollBox(Component).FControlList[j]).Width;
                          Instance.Parent := TControl(TEtwDBScrollBox(Component).FControlList[j]).Parent;

                          if (   (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBEdit)
                              or (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBComboBox)
                              or (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBSpinEdit)) then
                            begin
                              Datasource := TwwDBCustomEdit(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDatasource;
                              DataField  := TwwDBCustomEdit(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TwwDBCustomEdit(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TwwDBCustomEdit(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBComBoDlg) then
                            begin
                              Datasource := TwwDBComBoDlg(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TwwDBComBoDlg(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TwwDBComBoDlg(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TwwDBComboDlg(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBLookupCombo) then
                            begin
                              Datasource := TwwDBLookupCombo(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TwwDBLookupCombo(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TwwDBLookupCombo(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TwwDBLookupCombo(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBLookupComboDlg) then
                            begin
                              Datasource := TwwDBLookupComboDlg(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TwwDBLookupComboDlg(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TwwDBLookupComboDlg(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TwwDBLookupComboDlg(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TwwDBRichEdit) then
                            begin
                              Datasource := TwwDBRichEdit(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TwwDBRichEdit(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TwwDBRichEdit(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TwwDBRichEdit(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TDBImage) then
                            begin
                              Datasource := TDBImage(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TDBImage(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Order      := TDBImage(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TDBMemo) then
                            begin
                              Datasource := TDBMemo(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TDBMemo(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TDBMemo(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TDBMemo(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          if (TControl(TEtwDBScrollBox(Component).FControlList[j]) is TDBCheckBox) then
                            begin
                              Datasource := TDBCheckBox(TEtwDBScrollBox(Component).FControlList[j]).Datasource as TwwDataSource;
                              DataField  := TDBCheckBox(TEtwDBScrollBox(Component).FControlList[j]).DataField;
                              Color      := TDBCheckBox(TEtwDBScrollBox(Component).FControlList[j]).Color;
                              Order      := TDBCheckBox(TEtwDBScrollBox(Component).FControlList[j]).Taborder;
                            end;

                          TControl(TEtwDBScrollBox(Component).FControlList[j]).Free;
                          TEtwDBScrollBox(Component).FControlList.Delete(j);

                          case frmEdtDBScrollBox.rgpChangeTo.ItemIndex of
                            0 :
                              begin
                                TwwDBCustomEdit(Instance).Datasource := Datasource;
                                TwwDBCustomEdit(Instance).Datafield  := Datafield;
                                TwwDBCustomEdit(Instance).Color      := Color;
                                TwwDBCustomEdit(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.EditFields, i);
                              end;
                            1 :
                              begin
                                TwwDBCustomEdit(Instance).Datasource := Datasource;
                                TwwDBCustomEdit(Instance).Datafield  := Datafield;
                                TwwDBCustomEdit(Instance).Color      := Color;
                                TwwDBCustomEdit(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.ComboBoxFields, i);
                              end;
                            2 :
                              begin
                                TwwDBCustomEdit(Instance).Datasource := Datasource;
                                TwwDBCustomEdit(Instance).Datafield  := Datafield;
                                TwwDBCustomEdit(Instance).Color      := Color;
                                TwwDBCustomEdit(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.SpinEditFields, i);
                              end;


                            3 :
                              begin
                                TwwDBComBoDlg(Instance).Datasource := Datasource;
                                TwwDBComBoDlg(Instance).Datafield  := DataField;
                                TwwDBComboDlg(Instance).Color      := Color;
                                TwwDBComboDlg(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.ComboDlgFields, i);
                              end;
                            4 :
                              begin
                                TwwDBLookupCombo(Instance).Datasource := Datasource;
                                TwwDBLookupCombo(Instance).Datafield  := DataField;
                                TwwDBLookupCombo(Instance).Color      := Color;
                                TwwDBLookupCombo(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.LookupComboFields, i);
                              end;
                            5 :
                              begin
                                TwwDBLookupComboDlg(Instance).Datasource := Datasource;
                                TwwDBLookupComboDlg(Instance).Datafield  := DataField;
                                TwwDBLookupComboDlg(Instance).Color      := Color;
                                TwwDBLookupComboDlg(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.LookupComboDlgFields, i);
                              end;
                            6 :
                              begin
                                TwwDBRichEdit(Instance).Datasource := Datasource;
                                TwwDBRichEdit(Instance).Datafield  := Datafield;
                                TwwDBRichEdit(Instance).Color      := Color;
                                TwwDBRichEdit(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.MemoFields, i);
                              end;
                            7 :
                              begin
                                TDBImage(Instance).Datasource := Datasource;
                                TDBImage(Instance).Datafield  := Datafield;
                                TDBImage(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.ImageFields, i);
                              end;
                            8 :
                              begin
                                TDBMemo(Instance).Datasource := Datasource;
                                TDBMemo(Instance).Datafield  := Datafield;
                                TDBMemo(Instance).Color      := Color;
                                TDBMemo(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.MemoFields, i);
                              end;
                            9 :
                              begin
                                TDBCheckbox(Instance).Datasource := Datasource;
                                TDBCheckbox(Instance).Datafield  := Datafield;
                                TDBCheckbox(Instance).Color      := clBtnFace;
                                TDBCheckbox(Instance).Taborder   := Order;
                                TEtwDBScrollBox(Component).CreateUniqueName(Instance,
                                TEtwDBScrollBox(Component).FPrefix.CheckBoxFields, i);
                                TDBCheckbox(Instance).Caption := '';
                              end;
                          end;

                          Instance.Visible := true;

                          TEtwDBScrollBox(Component).FControlList.Insert(j, Instance);
                          TEtwDBScrollBox(Component).AlignLabelsAndFields;
                        end;
                    end;

                end;
            end;

        finally
          frmEdtDBScrollBox.Release;
        end;
      end;

    1:
      begin
        if not TEtwDBScrollBox(Component).MayEditProperties then
          begin
            raise exception.create('Overwriting the field order at this stage is not allowed');
          end;

        try

          frmEdtDBScrollBoxOrd             := TfrmEdtDBScrollBoxOrd.create(application);
          frmEdtDBScrollBoxOrd.ControlList := TList.Create;
          frmEdtDBScrollBoxOrd.FieldList   := TList.Create;

          for i := 0 to TEtwDBScrollBox(Component).FControlList.Count - 1 do
            begin
              if not (TControl(TEtwDBScrollBox(Component).FControlList.items[i]) is Tlabel) then
                begin
                  frmEdtDBScrollBoxOrd.AddItem(TControl(TEtwDBScrollBox(Component).FControlList[i]));
                end;
            end;

          if frmEdtDBScrollBoxOrd.lbxDBScrollBox.Items.Count = 0 then
            begin
              raise exception.create('Overwriting the field order at this stage is not allowed');
            end;

          for i := 0 to TEtwDBScrollBox(Component).FControlList.count - 1 do
            begin
              frmEdtDBScrollBoxOrd.ControlList.Add(TEtwDBScrollBox(Component).FControlList[i]);
            end;

          for i := 0 to TEtwDBScrollBox(Component).FFieldsList.count - 1 do
            begin
              frmEdtDBScrollBoxOrd.FieldList.Add(TEtwDBScrollBox(Component).FFieldsList[i]);
            end;

          if frmEdtDBScrollBoxOrd.ShowModal = mrOk then
            begin

              for i := TEtwDBScrollBox(Component).FControlList.count - 1 downto 0 do
                begin
                  TEtwDBScrollBox(Component).FControlList.delete(i);
                end;

              for i := TEtwDBScrollBox(Component).FFieldsList.count - 1 downto 0 do
                begin
                  TEtwDBScrollBox(Component).FFieldsList.delete(i);
                end;

              for i := 0 to frmEdtDBScrollBoxOrd.ControlList.count - 1 do
                begin
                  TEtwDBScrollBox(Component).FControlList.Add(frmEdtDBScrollBoxOrd.ControlList[i]);
                end;

              for i := 0 to frmEdtDBScrollBoxOrd.FieldList.count - 1 do
                begin
                  TEtwDBScrollBox(Component).FFieldsList.Add(frmEdtDBScrollBoxOrd.FieldList[i]);
                end;

              frmEdtDBScrollBoxOrd.ControlList.Free;
              frmEdtDBScrollBoxOrd.FieldList.Free;

              TEtwDBScrollBox(Component).AlignLabelsAndFields;
            end;
        finally
          frmEdtDBScrollBoxOrd.Release;
        end;
      end;
    2:   { Save properties }
      begin
        if messageDlg('Save properties for TEtwDBScrollBox ?',mtConfirmation,[mbYes,mbNo],0) = mrYes then
          begin
            GetWindowsDirectory(DirectoryArr,256);
            Directory := strPas(DirectoryArr);
            Ini       := TInifile.Create(Directory + '\DBScrollBox.INI');   // Create/Open INI-file

            try
              ini.writeInteger('Properties','EditSpacing',TEtwDBScrollBox(Component).FEditSpacing);
              ini.writeInteger('Properties','LabelAlign',ord(TEtwDBScrollBox(Component).FLabelAlign));
              ini.writeInteger('Properties','LabelAlignment',ord(TEtwDBScrollBox(Component).FLabelAlignment));
              ini.writeInteger('Properties','Labelspace',TEtwDBScrollBox(Component).FLabelspace);
              ini.writeBool   ('Properties','LabelUseSemiColon',TEtwDBScrollBox(Component).FLabelUseSemiColon);
              ini.writeBool   ('Properties','ConsistentEditWidth',TEtwDBScrollBox(Component).FConsistentEditWidth);
              ini.writeBool   ('Properties','LabelStripUnderscores',TEtwDBScrollBox(Component).FLabelStripUnderscores);
              ini.writeBool   ('Properties','LabelMixedCase',TEtwDBScrollBox(Component).FLabelMixedCase);
              ini.writeBool   ('Properties','LeftAlignmentTFields',TEtwDBScrollBox(Component).FLeftAlignmentTFields);
              ini.writeString ('Properties','PrefixEditFields',TEtwDBScrollBox(Component).FPrefix.EditFields);
              ini.writeString ('Properties','PrefixMemoFields',TEtwDBScrollBox(Component).FPrefix.MemoFields);
              ini.writeString ('Properties','PrefixImageFields',TEtwDBScrollBox(Component).FPrefix.ImageFields);
              ini.writeString ('Properties','PrefixComboBoxFields',TEtwDBScrollBox(Component).FPrefix.ComboBoxFields);
              ini.writeString ('Properties','PrefixComboDlgFields',TEtwDBScrollBox(Component).FPrefix.ComboDlgFields);
              ini.writeString ('Properties','PrefixLookupComboFields',TEtwDBScrollBox(Component).FPrefix.LookupComboFields);
              ini.writeString ('Properties','PrefixLookupComboDlgFields',TEtwDBScrollBox(Component).FPrefix.LookupComboDlgFields);
              ini.writeString ('Properties','PrefixSpinEditFields',TEtwDBScrollBox(Component).FPrefix.SpinEditFields);
              ini.writeString ('Properties','PrefixCheckBoxFields',TEtwDBScrollBox(Component).FPrefix.Checkboxfields);
              ini.writeString ('Properties','PrefixLabels',TEtwDBScrollBox(Component).FPrefix.Labels);
              ini.writeInteger('Properties','RequiredColor',TEtwDBScrollBox(Component).FRequiredColor);
            finally
              ini.Free;                                                  // Release
            end;
          end;
      end;
    3 :
      begin
        if not TEtwDBScrollBox(Component).MayEditProperties then
          begin
            raise exception.create('Overwriting properties at this stage not allowed');
          end;

        if messageDlg('Read properties for TEtwDBScrollBox ?',mtConfirmation,[mbYes,mbNo],0) = mrYes then
          begin
            GetWindowsDirectory(DirectoryArr,256);
            Directory := strPas(DirectoryArr);
            if FileExists(Directory + '\DBScrollBox.INI') then
              begin
                Ini := TInifile.Create(Directory + '\DBScrollBox.INI');   // Create/Open INI-file
                try
                  TEtwDBScrollBox(Component).EditSpacing             := ini.readInteger('Properties','EditSpacing',2);
                  TEtwDBScrollBox(Component).LabelSpace              := ini.readInteger('Properties','Labelspace',4);
                  TEtwDBScrollBox(Component).LabelAlign              := TetwLabelAlign(ini.readInteger('Properties','LabelAlign',1));
                  TEtwDBScrollBox(Component).LabelAlignment          := TAlignment(ini.readInteger('Properties','LabelAlignment',0));
                  TEtwDBScrollBox(Component).LabelUseSemiColon       := ini.readBool('Properties','LabelUseSemiColon',true);
                  TEtwDBScrollBox(Component).ConsistentEditWidth     := ini.readBool('Properties','ConsistentEditWidth',false);
                  TEtwDBScrollBox(Component).LabelStripUnderscores   := ini.readBool('Properties','LabelStripUnderscores',True);
                  TEtwDBScrollBox(Component).LabelMixedCase          := ini.readBool('Properties','LabelMixedCase',True);
                  TEtwDBScrollBox(Component).RequiredColor           := ini.readInteger('Properties','RequiredColor', clWindow);
                  TEtwDBScrollBox(Component).Prefix.EditFields       := ini.readString ('Properties','PrefixEditFields','edt');
                  TEtwDBScrollBox(Component).Prefix.MemoFields       := ini.readString ('Properties','PrefixMemoFields','mmo');
                  TEtwDBScrollBox(Component).Prefix.ImageFields      := ini.readString ('Properties','PrefixImageFields','img');
                  TEtwDBScrollBox(Component).Prefix.Labels           := ini.readString ('Properties','PrefixLabels','lbl');
                  TEtwDBScrollBox(Component).Prefix.ComboBoxFields   := ini.readString ('Properties','PrefixComboBoxFields','cbx');
                  TEtwDBScrollBox(Component).Prefix.ComboDlgFields   := ini.readString ('Properties','PrefixComboDlgFields','cdl');
                  TEtwDBScrollBox(Component).Prefix.LookupComboFields := ini.readString ('Properties','PrefixLookupComboFields','luc');
                  TEtwDBScrollBox(Component).Prefix.LookupComboDlgFields := ini.readString ('Properties','PrefixLookupComboDlgFields','lcd');
                  TEtwDBScrollBox(Component).Prefix.SpinEditFields   := ini.readString ('Properties','PrefixSpinEditFields','spn');
                  TEtwDBScrollBox(Component).Prefix.CheckboxFields   := ini.readString ('Properties','PrefixCheckboxFields','cbx');
                  TEtwDBScrollBox(Component).LeftAlignmentTFields    := ini.readBool('Properties','LeftAlignmentTFields',false);
                finally
                  Ini.Free;
                end;
              end
            else
              begin
                raise exception.create('DBScrollBox.INI does not exists');
              end;
          end;
      end;
    4 :
      begin
        if messageDlg('Do you want to replace TEtwDBScrollbox by TScrollBox', mtConfirmation,[mbYes,mbNo],0) <> mrYes then
            exit;

        ScrollBox        := TScrollBox.Create(TEtwDBScrollBox(Component).Owner);
        ScrollBox.Parent := TEtwDBScrollBox(Component).Parent;

        for i := TEtwDBScrollBox(Component).ControlCount - 1 downto 0 do
          begin
            TEtwDBScrollBox(Component).Controls[i].Parent := ScrollBox;
          end;

        Order := 0;
        for i := Scrollbox.ControlCount - 1  downto 0 do
          begin
            if (TControl(ScrollBox.Controls[i]) is TWinControl) then
              begin
                TWinControl(ScrollBox.Controls[i]).TabOrder := Order;
                inc(Order);
              end;
          end;


        with ScrollBox do
          begin
            Align          := TEtwDBScrollBox(Component).Align;
            AutoScroll     := TEtwDBScrollBox(Component).AutoScroll;
            BorderStyle    := TEtwDBScrollBox(Component).BorderStyle;
            Color          := TEtwDBScrollBox(Component).Color;
            Ctl3D          := TEtwDBScrollBox(Component).Ctl3D;
            Cursor         := TEtwDBScrollBox(Component).Cursor;
            DragCursor     := TEtwDBScrollBox(Component).DragCursor;
            DragMode       := TEtwDBScrollBox(Component).DragMode;
            Enabled        := TEtwDBScrollBox(Component).Enabled;
            Font           := TEtwDBScrollBox(Component).Font;
            Height         := TEtwDBScrollBox(Component).Height;
            HelpContext    := TEtwDBScrollBox(Component).HelpContext;
            Hint           := TEtwDBScrollBox(Component).Hint;
            HorzScrollBar  := TEtwDBScrollBox(Component).HorzScrollBar;
            Left           := TEtwDBScrollBox(Component).Left;
            Name           := TEtwDBScrollBox(Component).Name + 'etw';
            ParentColor    := TEtwDBScrollBox(Component).ParentColor;
            ParentCtl3D    := TEtwDBScrollBox(Component).ParentCtl3D;
            ParentFont     := TEtwDBScrollBox(Component).ParentFont;
            ParentShowHint := TEtwDBScrollBox(Component).ParentShowHint;
            PopupMenu      := TEtwDBScrollBox(Component).PopupMenu;
            ShowHint       := TEtwDBScrollBox(Component).ShowHint;
            TabOrder       := TEtwDBScrollBox(Component).TabOrder;
            TabStop        := TEtwDBScrollBox(Component).TabStop;
            Tag            := TEtwDBScrollBox(Component).Tag;
            Top            := TEtwDBScrollBox(Component).Top;
            VertScrollBar  := TEtwDBScrollBox(Component).VertScrollBar;
            Visible        := TEtwDBScrollBox(Component).Visible;
            Width          := TEtwDBScrollBox(Component).Width;
            OnClick        := TEtwDBScrollBox(Component).OnClick;
            OnDblClick     := TEtwDBScrollBox(Component).OnDblClick;
            OnDragDrop     := TEtwDBScrollBox(Component).OnDragDrop;
            OnDragOver     := TEtwDBScrollBox(Component).OnDragOver;
            OnEndDrag      := TEtwDBScrollBox(Component).OnEndDrag;
            OnEnter        := TEtwDBScrollBox(Component).OnEnter;
            OnExit         := TEtwDBScrollBox(Component).OnExit;
            OnMouseDown    := TEtwDBScrollBox(Component).OnMouseDown;
            OnMouseMove    := TEtwDBScrollBox(Component).OnMouseMove;
            OnMouseUp      := TEtwDBScrollBox(Component).OnMouseUp;
            OnResize       := TEtwDBScrollBox(Component).OnResize;
          end;

        TEtwDBScrollBox(Component).Free;

        ScrollBox.Name := system.copy(ScrollBox.Name ,1, (length(ScrollBox.Name) - 3));
      end;
  end;
  { case }

end;  { ExecuteVerb }

function  TEtwPrefix.ValidPrefix (aString : string) : boolean;
var
  i : integer;

begin
  result := true;

  for i := 1 to length(aString) do
    begin
      if (    ( (aString[i] >= 'a') and (aString[i] <= 'z') )
          or  ( (aString[i] >= 'A') and (aString[i] <= 'Z') ) ) then
        begin
        end
      else
        begin
          result := false;
          break;
        end;
    end;

end;

procedure TEtwPrefix.SetComboBoxFields (aString : string);
var
  i : integer;

begin

  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FComboBoxFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBComboBox) then
            begin
              if copy(TwwDBComboBox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FComboBoxFields)) = FComboBoxFields then
                begin
                  TwwDBComboBox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBComboBox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FComboBoxFields) + 1,
                    Length(TwwDBComboBox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FComboBoxFields));
                end;
            end;
        end;

      FComboBoxFields := aString;

    end;

end;

procedure TEtwPrefix.SetSpinEditFields (aString : string);
var
  i : integer;

begin

  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FSpinEditFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBSpinEdit) then
            begin
              if copy(TwwDBSpinEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FSpinEditFields)) = FSpinEditFields then
                begin
                  TwwDBSpinEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBSpinEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FSpinEditFields) + 1,
                    Length(TwwDBSpinEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FSpinEditFields));
                end;
            end;
        end;

      FSpinEditFields := aString;

    end;

end;

procedure TEtwPrefix.SetCheckboxFields (aString : string);
var
  i : integer;

begin

  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FCheckboxFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TDBCheckbox) then
            begin
              if copy(TwwDBSpinEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FCheckboxFields)) = FCheckboxFields then
                begin
                  TDBCheckbox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TDBCheckbox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FCheckboxFields) + 1,
                    Length(TDBCheckbox(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FCheckboxFields));
                end;
            end;
        end;

      FCheckboxFields := aString;

    end;

end;

procedure TEtwPrefix.SetComboDlgFields  (aString : string);
var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FComboDlgFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBComboDlg) then
            begin
              if copy(TwwDBComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FComboDlgFields)) = FComboDlgFields then
                begin
                  TwwDBComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FComboDlgFields) + 1,
                    Length(TwwDBComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FComboDlgFields));
                end;
            end;
        end;

      FComboDlgFields := aString;

    end;

end;

procedure TEtwPrefix.SetLookupComboFields    (aString : string);
var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLookupComboFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBLookupCombo) then
            begin
              if copy(TwwDBLookupCombo(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FLookupComboFields)) = FLookupComboFields then
                begin
                  TwwDBLookupCombo(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBLookupCombo(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FLookupComboFields) + 1,
                    Length(TwwDBLookupCombo(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FLookupComboFields));
                end;
            end;
        end;

      FLookupComboFields := aString;

    end;

end;

procedure TEtwPrefix.SetLookupComboDlgFields (aString : string);
var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLookupComboDlgFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBLookupComboDlg) then
            begin
              if copy(TwwDBLookupComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FLookupComboDlgFields)) = FLookupComboDlgFields then
                begin
                  TwwDBLookupComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBLookupComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FLookupComboDlgFields) + 1,
                    Length(TwwDBLookupComboDlg(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FLookupComboDlgFields));
                end;
            end;
        end;

      FLookupComboDlgFields := aString;

    end;

end;


procedure TEtwPrefix.SetEditFields ( aString : string);
var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FEditFields <> aString then
    begin

      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBEdit) then
            begin
              if copy(TwwDBedit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FEditFields)) = FEditFields then
                begin
                  TwwDBedit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBedit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FEditFields) + 1,
                    Length(TwwDBedit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FEditFields));
                end;
            end;
        end;
      FEditFields := aString;
    end;

end;

procedure TEtwPrefix.SetMemoFields ( aString : string);

var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FMemoFields <> aString then
    begin
      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i:= 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TwwDBRichEdit) then
            begin
              if copy(TwwDBRichEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FMemoFields)) = FMemoFields then
                begin
                  TwwDBRichEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TwwDBRichEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FMemoFields) + 1,
                                      Length(TwwDBRichEdit(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FMemoFields));
                end;
            end;
        end;
      FMemoFields := aString;
    end;


end;

procedure TEtwPrefix.SetImageFields ( aString : string);
var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FImageFields <> aString then
    begin
      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is TDBImage) then
            begin
              if copy(TDBImage(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FImageFields)) = FImageFields then
                begin
                  TDBImage(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(TDBImage(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FImageFields) + 1,
                                      Length(TDBImage(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FImageFields));
                end;
            end;
        end;
      FImageFields := aString;
    end;


end;

procedure TEtwPrefix.SetLabels ( aString : string);
var
  i : integer;

begin
  if not (csDesigning in TEtwDBScrollBox(FPrefixOwner).ComponentState) then
      exit;

  if not TEtwDBScrollBox(FPrefixOwner).MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabels <> aString then
    begin
      if not ValidPrefix (aString) then
        begin
          raise exception.create('Only alfanumeric characters allowed in prefixes');
        end;

      for i := 0 to TEtwDBScrollBox(FprefixOwner).FControlList.count - 1 do
        begin
          if (TControl(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]) is Tlabel) then
            begin
              if copy(Tlabel(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,1,Length(FLabels)) = FLabels then
                begin
                  Tlabel(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name :=
                   aString + copy(Tlabel(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name,Length(FLabels) + 1,
                                      Length(Tlabel(TEtwDBScrollBox(FprefixOwner).FControlList.items[i]).Name) - Length(FLabels));
                end;
            end;
        end;
      FLabels := aString;
    end;

end;

procedure TEtwDBScrollBox.Loaded;
begin
  inherited loaded;
  FMayEditProperties := false;
end;

procedure TEtwDBScrollBox.ReadIniFile;
var
  directoryArr : array [0..255] of char;
  directory    : string;
  ini          : TIniFile;
begin

  GetWindowsDirectory(DirectoryArr,256);
  Directory := strPas(DirectoryArr);

  if FileExists(Directory + '\DBScrollBox.INI') then
    begin
      Ini := TInifile.Create(Directory + '\DBScrollBox.INI');   // Create/Open INI-file
      try
        EditSpacing                 := ini.readInteger('Properties','EditSpacing',2);
        LabelSpace                  := ini.readInteger('Properties','Labelspace',4);
        LabelAlign                  := TetwLabelAlign(ini.readInteger('Properties','LabelAlign',1));
        LabelAlignment              := TAlignment(ini.readInteger('Properties','LabelAlignment',0));
        LabelUseSemiColon           := ini.readBool('Properties','LabelUseSemiColon',false);
        ConsistentEditWidth         := ini.readBool('Properties','ConsistentEditWidth',false);
        LabelStripUnderscores       := ini.readBool('Properties','LabelStripUnderscores',false);
        LabelMixedCase              := ini.readBool('Properties','LabelMixedCase',false);
        RequiredColor               := ini.readInteger('Properties','RequiredColor', clWindow);
        Prefix.EditFields           := ini.readString ('Properties','PrefixEditFields','edt');
        Prefix.MemoFields           := ini.readString ('Properties','PrefixMemoFields','mmo');
        Prefix.ImageFields          := ini.readString ('Properties','PrefixImageFields','img');
        Prefix.Labels               := ini.readString ('Properties','PrefixLabels','lbl');
        Prefix.ComboBoxFields       := ini.readString ('Properties','PrefixComboBoxFields','cbx');
        Prefix.ComboDlgFields       := ini.readString ('Properties','PrefixComboDlgFields','cdl');
        Prefix.LookupComboFields    := ini.readString ('Properties','PrefixLookupComboFields','luc');
        Prefix.LookupComboDlgFields := ini.readString ('Properties','PrefixLookupComboDlgFields','lcd');
        Prefix.SpinEditFields       := ini.readString ('Properties','PrefixSpinEditFields','spn');
        Prefix.CheckboxFields       := ini.readString ('Properties','PrefixCheckboxFields','cbx');
        LeftAlignmentTFields        := ini.readBool('Properties','LeftAlignmentTFields',false);
      finally
        Ini.Free;
      end;
    end;

end;

procedure TEtwDBScrollBox.AlignLabelsAndFields;
var
  i             : integer;
  MaxLabelWidth : integer;
  EditTop       : integer;

begin
  MaxLabelWidth := -1;

  for i := 0 to FControlList.Count - 1 do
    begin
      if TControl(FControlList.items[i]) is TLabel then
        begin
            begin
              TLabel(FControlList.items[i]).AutoSize := true;
              if TLabel(FControlList.items[i]).Width > MaxLabelWidth then
                begin
                  MaxLabelWidth := TLabel(FControlList.items[i]).Width;
                end;
            end;
        end;
    end;

  for i := 0 to FControlList.Count - 1 do
    begin
      if TControl(FControlList.items[i]) is TLabel then
        begin
            begin
              TLabel(FControlList.items[i]).AutoSize := false;
              TLabel(FControlList.items[i]).Width    := MaxLabelWidth;
            end;
        end;
    end;

  EditTop := 4;

  for i:= 0 to FControlList.count - 1 do
    begin

      if FLabelAlign = alLeft then
        begin

          if not TControl(FControlList.items[i]).Visible then
            begin
              TControl(FControlList.items[i]).Visible := true;
            end;

          if (TControl(FControlList.items[i]) is TLabel) then
            begin
              TControl(FControlList.items[i]).Left    := 10;
              TLabel(FControlList.items[i]).Alignment := FLabelAlignment;

              if (    (TControl(FControlList.items[i + 1]) is TDBImage)
                  or  (TControl(FControlList.items[i + 1]) is TDBMemo)
                  or  (TControl(FControlList.items[i + 1]) is TwwDBRichEdit)) then
                begin
                  TControl(FControlList.items[i]).Top := EditTop + 2;
                end
              else
                begin
                  TControl(FControlList.items[i]).Top := EditTop + (TControl(FControlList.items[i + 1]).Height div 2)
                     - ( TControl(FControlList.items[i]).Height div 2);
                end;
            end
          else
            begin
              TControl(FControlList.items[i]).Left := MaxLabelWidth + 10 + FLabelSpace;
              TControl(FControlList.items[i]).Top  := EditTop;
              EditTop                              := EditTop + TControl(FControlList.items[i]).Height + FEditSpacing;
            end;

        end
      else
        begin

          if not TControl(FControlList.items[i]).Visible then
            begin
              TControl(FControlList.items[i]).Visible := true;
            end;

          TControl(FControlList.items[i]).Top  := EditTop;
          TControl(FControlList.items[i]).Left := 10;

          if (TControl(FControlList.items[i]) is TLabel) then
            begin
              TLabel(FControlList.items[i]).Alignment := FLabelAlignment;
              EditTop                                 := EditTop + TLabel(FControlList.items[i]).Height +  FLabelSpace;
            end
          else
            begin
              EditTop := EditTop + TControl(FControlList.items[i]).Height + FEditSpacing;
            end;
        end;
    end;

end;

procedure TEtwDBScrollBox.notification (AComponent : TComponent; Operation : TOperation);
var
  i : integer;
  //  MyControl : TControl;
begin
  inherited Notification(AComponent, Operation);

  if not (csDesigning in ComponentState) then
      exit;

  if (( Operation = opRemove ) and (AComponent = FDataSource)) then
    begin

      for i := FFieldsList.count - 1 downto 0 do
        begin
          FFieldsList.Delete(i);
        end;


      FDataSource := nil;

    end;

  if (Operation = opRemove ) then
    begin
      for i := 0 to FControlList.count - 1 do
        begin
          if (AComponent.Name = TControl(FControlList.items[i]).Name) then
            begin
              if FComingFromTheEditor then
                begin
                  Dec(FNumberOfEditorChanges);
                  if FNumberOfEditorChanges = 0 then
                    begin
                      FComingFromTheEditor := false;
                    end;
                  break;
                end
              else
                begin
                  //                  if TControl(FControlList.items[i]) is TLabel then begin
                  //                    MyControl := TControl(FControlList.items[i + 1]);
                  //                    FControlList.delete(i * 2);      // delete the entry of the Label
                  //                    FControlList.delete(i * 2);     // delete the entry of the editfield
                  //                    MyControl.Free;
                  //                  end else begin
                  //                  end;
                  //                  AlignLabelsAndFields;
                  FMayEditProperties := false;
                  Break;
                end;
            end;
        end;
    end;

end;

procedure TEtwDBScrollBox.SetRequiredColor  ( aColor : TColor);
var
  j : integer;

begin

  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if (FRequiredColor <> aColor) then
    begin

      FRequiredColor := aColor;

      for j := 0 to FControlList.count - 1 do
        begin

          if not (TControl(FControlList[j]) is TLabel) then
            begin
              if (TField(FFieldsList.items[(j div 2)]).Required) then
                begin
                  if (   (TControl(FControlList[j]) is TwwDBEdit)
                      or (TControl(FControlList[j]) is TwwDBComboBox)
                      or (TControl(FControlList[j]) is TwwDBSpinEdit)) then
                    begin
                      TwwDBCustomEdit(FControlList[j]).Color := FRequiredColor;
                    end;

                  if (TControl(FControlList[j]) is TwwDBComBoDlg) then
                    begin
                      TwwDBComboDlg(FControlList[j]).Color := FRequiredColor;
                    end;

                  if (TControl(FControlList[j]) is TwwDBLookupComBoDlg) then
                    begin
                      TwwDBLookUpComboDlg(FControlList[j]).Color := FRequiredColor;
                    end;

                  if (TControl(FControlList[j]) is TwwDBLookUpComBo) then
                    begin
                      TwwDBLookUpComBo(FControlList[j]).Color := FRequiredColor;
                    end;

                  if (TControl(FControlList[j]) is TwwDBRichEdit) then
                    begin
                      TwwDBRichEdit(FControlList[j]).Color := FRequiredColor;
                    end;

                  if (TControl(FControlList[j]) is TDBMemo) then
                    begin
                      TDBMemo(FControlList[j]).Color := FRequiredColor;
                    end;

                  if (TControl(FControlList[j]) is TDBCheckbox) then
                    begin
                      TDBCheckbox(FControlList[j]).Color := FRequiredColor;
                    end;

                end;

            end;

        end;

    end;

end;

procedure TEtwDBScrollBox.SetLeftAlignmentTFields (aBoolean : boolean);
var
  i : integer;
begin

  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLeftAlignmentTFields <> aBoolean then
    begin
      FLeftAlignmentTFields := aBoolean;

      if (    (FDatasource <> nil )
                and (FDatasource.dataset <> nil)) then
        begin

          with FDataSource.Dataset do
            begin
              for  i := 0 to (fieldCount - 1) do
                begin
                  try
                    if FLeftAlignmentTFields then
                      begin
                        fields[i].alignment := taLeftJustify;
                      end
                    else
                      begin

                        case fields[i].datatype of
                          ftSmallint 	  :
                              fields[i].alignment := taRightJustify;
                          ftInteger 	  :
                              fields[i].alignment := taRightJustify;
                          ftWord 		    :
                              fields[i].alignment := taRightJustify;
                          ftFloat 		  :
                              fields[i].alignment := taRightJustify;
                          ftCurrency 	  :
                              fields[i].alignment := taRightJustify;
                          ftBCD			    :
                              fields[i].alignment := taRightJustify;
                          ftAutoInc	    :
                              fields[i].alignment := taRightJustify;
                          ftTypedBinary :
                              fields[i].alignment := taRightJustify;
                        else
                          begin
                            fields[i].alignment := taLeftJustify;
                          end;
                        end;
                      end;
                  except
                  end;
                end;
            end;
        end;
    end;

end;

procedure TEtwDBScrollBox.SetLabelAlign (aEtwLabelAlign : TEtwLabelAlign);
begin
  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabelAlign <> aEtwLabelAlign then
    begin
      FLabelAlign := aEtwLabelAlign;
      AlignLabelsAndFields;
    end;
end;

procedure TEtwDBScrollBox.SetLabelSpace (aInteger : integer);
begin
  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabelSpace <> aInteger then
    begin
      FLabelSpace := aInteger;
      AlignLabelsAndFields;
    end;
end;

function TEtwDBScrollBox.CreateCaption ( aInteger : integer ) : string;

var
  i               : integer;
  NewDisplayLabel : string;
  Upper           : boolean;
  strUpper        : string;

begin

  NewDisplayLabel := TField(FFieldsList.Items[aInteger]).DisplayName;

  if FLabelUseSemiColon then
    begin
      NewdisplayLabel := NewDisplayLabel + ' :'
    end;

  if FLabelMixedCase then
    begin
      upper := true;

      for i := 1 to Length(NewDisplayLabel) do
        begin
          if Upper then
            begin
              strUpper           := NewDisplayLabel[i];
              strUpper           := UpperCase(strUpper);
              NewDisplayLabel[i] := strUpper[1];
            end
          else
            begin
              strUpper           := NewDisplayLabel[i];
              strUpper           := LowerCase(strUpper);
              NewDisplayLabel[i] := strUpper[1];
            end;

          Upper := (  (NewDisplayLabel[i] = '_')
                        or (NewDisplayLabel[i] = ' '));
        end;
    end;

  if FLabelStripUnderscores then
    begin
      for i := 1 to Length(NewDisplayLabel) do
        begin
          if NewDisplayLabel[i] = '_' then
            begin
              NewDisplayLabel[i] := ' ';
            end;
        end;
    end;

  result := NewDisplayLabel;

end;

procedure TEtwDBScrollBox.SetLabelMixedCase (aBoolean : boolean);
var
  i : integer;

begin
  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabelMixedCase <> aBoolean then
    begin
      if DataSource <> nil then
        begin
          FLabelMixedCase := aBoolean;

          for i := 0 to FControlList.count - 1 do
            begin
              if TControl(FControlList.items[i]) is TLabel then
                begin
                  TLabel(FControlList.items[i]).caption := CreateCaption(i div 2);
                end;
            end;
          AlignLabelsAndFields;
        end
      else
        begin
          FLabelMixedCase := aBoolean;
        end;
    end;
end;

procedure TEtwDBScrollBox.SetLabelStripUnderscores (aBoolean : boolean);
var
  i : integer;
begin
  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabelStripUnderscores <> aBoolean then
    begin
      if DataSource <> nil then
        begin
          FLabelStripUnderscores := aBoolean;

          for i:= 0 to FControlList.count - 1 do
            begin
              if TControl(FControlList.items[i]) is TLabel then
                begin
                  TLabel(FControlList.items[i]).caption := CreateCaption(I div 2);
                end;
            end;
          AlignLabelsAndFields;
        end
      else
        begin
          FLabelStripUnderscores := aBoolean;
        end;
    end;
end;

procedure TEtwDBScrollBox.SetEditSpacing (aInteger : integer);
begin
  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FEditSpacing <> aInteger then
    begin
      FEditSpacing := aInteger;
      AlignLabelsAndFields;
    end;

end;

procedure TEtwDBScrollBox.SetConsistentEditWidth (aBoolean : boolean);
var
  i            : integer;
  MaxEditWidth : integer;
begin

  MaxEditWidth := -1;

  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FConsistentEditWidth <> aBoolean then
    begin
      FConsistentEditWidth := aBoolean;

      if FConsistentEditWidth then   // calculate the maximum width
        begin

          for i:= 0 to FControlList.count - 1 do
            begin

              if not (TControl(FControlList.items[i]) is TLabel) then // not the TLabel's.
                begin
                  if TControl(FControlList.items[i]).width > MaxEditWidth then
                    begin
                      MaxEditWidth := TControl(FControlList.items[i]).width;
                    end;
                end;
            end;
        end;


      for i := 0 to FControlList.count - 1 do
        begin
          if not (TControl(FControlList.items[i]) is TLabel) then
            begin
              if FConsistentEditWidth then
                begin
                  TControl(FControlList.items[i]).width := MaxEditWidth;
                end
              else
                begin
                  TControl(FControlList.items[i]).width :=
                   (TField(FFieldsList.items[(i div 2)]).DisplayWidth + 1) * FCharWidth  + 3;
                end;
            end;
        end;
    end;

end;

procedure TEtwDBScrollBox.SetLabelAlignment (aAlignment : TAlignment);
var
  i : integer;
begin
  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabelAlignment <> aAlignment then
    begin
      FLabelAlignment := aAlignment;
      for i := 0 to FControlList.count - 1 do
        begin
          if (TControl(FControlList.items[i]) is TLabel) then
            begin
              TLabel(FControlList.items[i]).Alignment := FLabelAlignment;
            end;
        end;
    end;

end;

procedure TEtwDBScrollBox.SetLabelUseSemiColon (aBoolean : Boolean);
var
  i : integer;
begin
  if not (csDesigning in ComponentState) then
      exit;

  if not MayEditProperties then
    begin
      raise exception.create('Overwriting this property at this stage is not allowed');
    end;

  if FLabelUseSemiColon  <> aBoolean then
    begin
      FLabelUseSemiColon := aBoolean;


      for i:= 0 to FControlList.count - 1 do
        begin
          if (TControl(FControlList.items[i]) is TLabel) then
            begin
              if LabelUseSemiColon then
                begin
                  TLabel(FControlList.items[i]).Caption := TLabel(FControlList.items[i]).Caption + ' :';
                end
              else
                begin
                  TLabel(FControlList.items[i]).Caption :=
                   Copy( TLabel(FControlList.items[i]).Caption, 1,
                        Length(TLabel(FControlList.items[i]).Caption) - 2 );
                end;
            end;
        end;

      AlignLabelsAndFields;
    end;

end;

function TEtwDBScrollBox.GetDatasource: TwwDatasource;  { Returns linked data source. }
begin
  { FFieldDataLink is built in TEtwDBScrollBox.Create; there's no need to check to see if it's assigned. }
  GetDatasource := FDataSource;
end;		{ GetDatasource }

procedure TEtwDBScrollBox.CreateUniqueName (aControl : TControl; prefix : string; i : integer);

  function ReplaceInvalidCharacters (s : string) : string;

  var
    j : integer;
    r : string;

  begin
    r := '';

    for j := 1 to length(s) do
      begin
        try
          if (    ( (s[j] >= 'a') and (s[j] <= 'z') )
              or  ( (s[j] >= 'A') and (s[j] <= 'Z') )
              or  ( (s[j] >= '0') and (s[j] <= '9') )
              ) then
            begin
              r := r + s[j];
            end;
        except
          Raise;
        end;

      end;

    result := r;
  end;

var
  j          : integer;
  uniqueName : boolean;
  NewName    : string;
begin

  uniqueName := false;   // generate unique name
  J          := 0;

  while not UniqueName do
    begin
      try
        if J = 0 then
          begin
            newName := trim(Prefix) + ReplaceInvalidCharacters(TField(FFieldsList.items[i]).fieldname);
          end
        else
          begin
            newName := trim(Prefix) + ReplaceInvalidCharacters(TField(FFieldsList.items[i]).fieldname) + IntToStr(j);
          end;
        AControl.Name := newName;
        UniqueName    := true;
      except
        inc(j);
        if J > 99 then
            raise;
      end;
    end;
end;


procedure TEtwDBScrollBox.SetDatasource(newSource: TwwDataSource);  { Assigns new data source. }

var
  i                 : integer;
  NewLabel          : TLabel;
  wwDBEdit          : TwwDBEdit;
  wwMemo            : TwwDBRichEdit;
  Image             : TDBImage;
  DataSourceChanged : boolean;

begin

  if not (csDesigning in Componentstate) then
      exit; // Only in Design mode

  if not FMayEditProperties then
    begin
      if FDatasource <> nil then
          exit;
    end;

  DatasourceChanged := false;

  if (FDatasource = newSource) then
      exit
  else
    begin

      if (     (FControlList <> nil)
           and (FControlList.Count > 0)) then  // Clear ControlList
        begin
          DataSourceChanged := true;


          for i := FControlList.count - 1 downto 0 do
            begin
              try
                TControl(FControlList.items[i]).Free;
              except
              end;
              FControlList.Delete(i);
            end;

          for i := FFieldsList.count - 1 downto 0 do
            begin
              FFieldsList.Delete(i);
            end;
        end;

      FDatasource := NewSource;

      if FDatasource <> nil then
          Self.FreeNotification (FDatasource);
    end;

  if FDatasource = nil then
      exit // Dataset value filled ??
  ;

  if FDatasource.dataset = nil then
      exit // Dataset value filled ??
  ;

  if (csLoading in Componentstate) then
      exit; // Only in Design mode

  if not FDataSource.Dataset.active then
      FDataSource.Dataset.active := true;

  with FDataSource.Dataset do
    begin
      for  i := 0 to (fieldCount - 1) do
        begin

          if FLeftAlignmentTFields then
            begin
              fields[i].alignment := taLeftJustify;
            end;

          FFieldsList.Add(fields[i]);

          NewLabel := TLabel.Create(Self.owner);

          with NewLabel do
            begin
              parent   := self;
              visible  := false;
              Autosize := true;
              CreateUniqueName(NewLabel,FPrefix.Labels, i);

              alignment := FLabelAlignment;

              caption := CreateCaption (i);


            end;
          // With NewLabel do

          FControlList.Add(NewLabel);

          if ((fields[i].dataType <> ftMemo) and (fields[i].datatype <> ftGraphic)) then
            begin
              wwDBEdit := TwwDBEdit.create(Self.Owner);

              with wwDBEdit do
                begin
                  parent  := self;
                  visible := false;

                  if fields[i].required then
                    begin
                      color := FRequiredColor;
                    end;

                  CreateUniqueName(wwDBEdit,FPrefix.EditFields, i);

                  DataSource := self.datasource as TwwDatasource;
                  DataField  := fields[i].fieldname;
                  width      := (fields[i].displayWidth + 1) * FCharWidth  + 3;

                  TLabel(FControlList.items[FControlList.Count - 1]).focusControl := wwDBEdit;
                end;

              FControlList.add(wwDBEdit);
            end
          else
            begin
              if fields[i].dataType = ftMemo then
                begin
                  wwMemo := TwwDBRichEdit.Create(self.owner);
                  with wwMemo do
                    begin
                      parent  := self;
                      visible := false;
                      CreateUniqueName(wwMemo,FPrefix.MemoFields, i);

                      DataSource := self.datasource as TwwDatasource;
                      DataField  := fields[i].fieldname;
                      width      := (fields[i].displayWidth + 1) * FCharWidth  + 3;

                      TLabel(FControlList.items[FControlList.Count - 1]).focusControl := wwMemo;
                    end;

                  FControlList.add(wwMemo);
                end
              else
                begin
                  Image := TDBImage.Create(self.owner);

                  with Image do
                    begin
                      parent     := self;
                      visible    := false;
                      stretch    := true;
                      DataSource := self.datasource as TwwDatasource;
                      DataField  := fields[i].fieldname;
                      width      := (fields[i].displayWidth + 1) * FCharWidth  + 3;

                      CreateUniqueName(Image,FPrefix.ImageFields, i);

                      TLabel(FControlList.items[FControlList.Count - 1]).focusControl := Image;
                    end;

                  FControlList.add(Image);
                end;
            end;
        end;
      // End for  i:= 0 to fieldCount

      if (FControlList.Count > 0) then
        begin
          FMayEditProperties := true;

          if not DatasourceChanged then
              ReadIniFile;

          AlignLabelsAndFields;
        end;


    end;
  // with DataSource.Dataset


end;		{ SeTDatasource }

destructor TEtwDBScrollBox.Destroy;

begin

  if (csDesigning in ComponentState) then
    begin
      FControlList.Free;
      FFieldsList.Free;
    end;

  FPrefix.Free;
  inherited Destroy;
end;  { Destroy }

constructor TEtwDBScrollBox.Create(AOwner: TComponent);
{ Creates an object of type TEtwDBScrollBox, and initializes properties. }
begin

  inherited Create(AOwner);
  BorderStyle := bsNone;
  Align       := alClient;


  with self.owner as TForm do
    begin
      FCharWidth      := Canvas.TextWidth('0');
      FSemiColonWidth := Canvas.TextWidth(' :');
    end;

  FComingFromTheEditor   := false;
  FNumberOfEditorChanges := 0;
  FLabelUseSemiColon     := false;
  FConsistentEditWidth   := false;
  FLabelMixedCase        := false;
  FEditSpacing           := 2;
  FLabelAlignment        := taLeftJustify;
  FLabelAlign            := alLeft;
  FLabelStripUnderscores := false;
  FLabelSpace            := 4;
  FLeftAlignmentTFields  := false;
  FRequiredColor         := clWindow;

  FPrefix := TEtwPrefix.CreateFor(Self);

  Prefix.FEditFields           := 'edt';
  Prefix.FMemoFields           := 'mmo';
  Prefix.FImageFields          := 'img';
  Prefix.FLabels               := 'lbl';
  Prefix.FComboBoxFields       := 'cbx';
  Prefix.FSpinEditFields       := 'spn';
  Prefix.FComboDlgFields       := 'cdl';
  Prefix.FLookupComboFields    := 'luc';
  Prefix.FLookupComboDlgFields := 'lcd';
  Prefix.FCheckboxFields       := 'cbx';

  FMayEditProperties := true;

  if (csDesigning in ComponentState) then
    begin
      FControlList := TList.create;
      FFieldsList  := TList.create;
    end;

end;  { Create }

procedure Register;
begin

  RegisterComponents('EtW', [TEtwDBScrollBox]);
  RegisterComponentEditor(TEtwDBScrollBox, TEtwDBScrollBoxEditor);
  RegisterPropertyEditor( TypeInfo(TEtWAboutInfo), nil, 'About',TEtWAboutBoxProperty);

end;  { Register }

end.

